@@ -45,6 +45,7 @@
 #include "calendar-window.h"
 
 #include "clock.h"
+#include "clock-utils.h"
 #include "clock-typebuiltins.h"
 #ifdef HAVE_LIBECAL
 #include "calendar-client.h"
@@ -55,10 +56,10 @@
 #ifdef HAVE_LIBECAL
 #define N_CALENDAR_WINDOW_GCONF_PREFS 4
 
-static const char* KEY_APPOINTMENTS_EXPANDED = "expand_appointments";
-static const char* KEY_BIRTHDAYS_EXPANDED    = "expand_birthdays";
-static const char* KEY_TASKS_EXPANDED        = "expand_tasks";
-static const char* KEY_WEATHER_EXPANDED      = "expand_weather";
+#define KEY_APPOINTMENTS_EXPANDED "expand_appointments"
+#define KEY_BIRTHDAYS_EXPANDED    "expand_birthdays"
+#define KEY_TASKS_EXPANDED        "expand_tasks"
+#define KEY_WEATHER_EXPANDED      "expand_weather"
 #endif
 
 struct _CalendarWindowPrivate {
@@ -649,17 +650,27 @@ compare_tasks  (GtkTreeModel *model,
                 GtkTreeIter  *b,
                 gpointer      user_data)
 {
-        int priority_a;
-        int priority_b;
+        gboolean done_a, done_b;
+        int priority_a, priority_b;
 
-        gtk_tree_model_get (model, a, TASK_COLUMN_PRIORITY, &priority_a, -1);
-        gtk_tree_model_get (model, b, TASK_COLUMN_PRIORITY, &priority_b, -1);
+        gtk_tree_model_get (model, a,
+                            TASK_COLUMN_COMPLETED, &done_a,
+                            TASK_COLUMN_PRIORITY, &priority_a,
+                            -1);
+        gtk_tree_model_get (model, b,
+                            TASK_COLUMN_COMPLETED, &done_b,
+                            TASK_COLUMN_PRIORITY, &priority_b,
+                            -1);
 
-	/* We change undefined priorities so they appear after 'Low'. */
-	if (priority_a <= 0)
-		priority_a = 10;
-	if (priority_b <= 0)
-		priority_b = 10;
+        /* Always sort completed tasks last */
+        if (done_a != done_b)
+                return done_a ? -1 : 1;
+
+        /* We change undefined priorities so they appear as "Normal" */
+        if (priority_a <= 0)
+                priority_a = 5;
+        if (priority_b <= 0)
+                priority_b = 5;
 
 	/* We'll just use the ordering of the priority values. */
 	if (priority_a < priority_b)
@@ -678,8 +689,21 @@ compare_tasks  (GtkTreeModel *model,
                         return -1;
                 else if (due_time_a > due_time_b)
                         return 1;
-                else
-                        return 0;
+                else {
+                        char *summary_a, *summary_b;
+                        int res;
+
+                        gtk_tree_model_get (model, a, TASK_COLUMN_SUMMARY, &summary_a, -1);
+                        gtk_tree_model_get (model, b, TASK_COLUMN_SUMMARY, &summary_b, -1);
+
+                        res = g_utf8_collate (summary_a ? summary_a: "",
+                                              summary_b ? summary_b: "");
+
+                        g_free (summary_a);
+                        g_free (summary_b);
+
+                        return res;
+                }
         }
 }
 
@@ -1568,6 +1592,10 @@ calendar_window_destroy (GtkObject *object)
 						    calwin->priv->listeners [i]);
 		calwin->priv->listeners [i] = 0;
 	}
+
+	if (calwin->priv->gconfclient)
+		g_object_unref (calwin->priv->gconfclient);
+	calwin->priv->gconfclient = NULL;
 #endif /* HAVE_LIBECAL */
 
 	GTK_OBJECT_CLASS (calendar_window_parent_class)->destroy (object);
