@@ -9,6 +9,7 @@
 #define PANEL_ICON_FOLDER		"folder"
 #define PANEL_ICON_FONTS		"preferences-desktop-font"
 #define PANEL_ICON_FORCE_QUIT		"gnome-panel-force-quit"
+#define PANEL_ICON_HOME			"user-home"
 #define PANEL_ICON_LAUNCHER		"gnome-panel-launcher"
 #define PANEL_ICON_LOCKSCREEN		"system-lock-screen"
 #define PANEL_ICON_LOGOUT		"gnome-logout"
@@ -16,6 +17,7 @@
 #define PANEL_ICON_NETWORK		"network-workgroup"
 #define PANEL_ICON_NETWORK_SERVER	"network-server"
 #define PANEL_ICON_PANEL		"gnome-panel"
+#define PANEL_ICON_RECENT		"document-open-recent"
 #define PANEL_ICON_REMOTE		"applications-internet"
 #define PANEL_ICON_REMOVABLE_MEDIA	"drive-removable-media"
 #define PANEL_ICON_RUN			"system-run"
