@@ -128,6 +128,13 @@ activate_uri (GtkWidget  *menuitem,
 	g_free (url);
 }
 
+static void
+activate_home_uri (GtkWidget *menuitem,
+		   gpointer   data)
+{
+	activate_uri (menuitem, g_get_home_dir ());
+}
+
 static void
 activate_desktop_uri (GtkWidget *menuitem,
 		      gpointer   data)
@@ -199,10 +206,9 @@ panel_menu_items_append_from_desktop (GtkWidget *menu,
 
 	item = gtk_image_menu_item_new ();
 	setup_menu_item_with_icon (item, panel_menu_icon_get_size (),
-				   icon, NULL, name, TRUE);
+				   icon, NULL, name);
 
-	if (comment != NULL)
-		gtk_tooltips_set_tip (panel_tooltips, item, comment, NULL);
+	panel_util_set_tooltip_text (item, comment);
 
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 	g_signal_connect_data (item, "activate",
@@ -247,13 +253,9 @@ panel_menu_items_append_place_item (const char *icon_name,
 				   panel_menu_icon_get_size (),
 				   icon_name,
 				   NULL,
-				   title,
-				   TRUE);
+				   title);
 
-	gtk_tooltips_set_tip (panel_tooltips,
-			      item,
-			      tooltip,
-			      NULL);
+	panel_util_set_tooltip_text (item, tooltip);
 
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 
@@ -282,13 +284,12 @@ panel_menu_items_create_action_item_full (PanelActionButtonType  action_type,
 				   panel_menu_icon_get_size (),
 				   panel_action_get_icon_name (action_type),
 				   NULL,
-				   label ? label : panel_action_get_text (action_type),
-				   TRUE);
+				   label ? label : panel_action_get_text (action_type));
 
-	gtk_tooltips_set_tip (panel_tooltips,
-			      item,
-			      tooltip ? tooltip : panel_action_get_tooltip (action_type),
-			      NULL);
+	panel_util_set_tooltip_text (item,
+				     tooltip ?
+					tooltip :
+					panel_action_get_tooltip (action_type));
 
 	g_signal_connect (item, "activate",
 			  panel_action_get_invoke (action_type), NULL);
@@ -401,7 +402,7 @@ panel_place_menu_item_append_gtk_bookmarks (GtkWidget *menu)
 		item = gtk_image_menu_item_new ();
 		setup_menu_item_with_icon (item, panel_menu_icon_get_size (),
 					   PANEL_ICON_BOOKMARKS, NULL,
-					   _("Bookmarks"), TRUE);
+					   _("Bookmarks"));
 
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 		gtk_widget_show (item);
@@ -521,7 +522,7 @@ panel_place_menu_item_append_volumes (GtkWidget *menu,
 
 		item = gtk_image_menu_item_new ();
 		setup_menu_item_with_icon (item, panel_menu_icon_get_size (),
-					   icon, NULL, title, TRUE);
+					   icon, NULL, title);
 
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 		gtk_widget_show (item);
@@ -565,21 +566,31 @@ panel_place_menu_item_create_menu (PanelPlaceMenuItem *place_item)
 	GtkWidget *places_menu;
 	GtkWidget *item;
 	char      *gconf_name;
+	char      *uri;
 
 	places_menu = panel_create_menu ();
 
 	gconf_name = gconf_client_get_string (panel_gconf_get_client (),
 					      HOME_NAME_KEY,
 					      NULL);
-	panel_menu_items_append_from_desktop (places_menu,
-					      "nautilus-home.desktop",
-					      gconf_name);
+
+	uri = gnome_vfs_get_uri_from_local_path (g_get_home_dir ());
+	panel_menu_items_append_place_item (PANEL_ICON_HOME,
+					    string_empty (gconf_name) ?
+						_("Home Folder") : gconf_name,
+					    _("Open your personal folder"),
+					    places_menu,
+					    G_CALLBACK (activate_home_uri),
+					    uri);
+	g_free (uri);
+
 	if (gconf_name)
 		g_free (gconf_name);
 
 	if (!gconf_client_get_bool (panel_gconf_get_client (),
 				    DESKTOP_IS_HOME_DIR_KEY,
 				    NULL)) {
+		uri = gnome_vfs_get_uri_from_local_path (g_get_user_special_dir (G_USER_DIRECTORY_DESKTOP));
 		panel_menu_items_append_place_item (
 				PANEL_ICON_DESKTOP,
 				/* Translators: Desktop is used here as in
@@ -590,7 +601,9 @@ panel_place_menu_item_create_menu (PanelPlaceMenuItem *place_item)
 				_("Open the contents of your desktop in a folder"),
 				places_menu,
 				G_CALLBACK (activate_desktop_uri),
-				NULL);
+				/* FIXME: if the dir changes, we'd need to update the drag data since the uri is not the same */
+				uri);
+		g_free (uri);
 	}
 
 	panel_place_menu_item_append_gtk_bookmarks (places_menu);
@@ -895,8 +908,7 @@ panel_place_menu_item_new (gboolean use_image)
 	setup_menuitem (GTK_WIDGET (menuitem),
 			image ? panel_menu_icon_get_size () : GTK_ICON_SIZE_INVALID,
 			image,
-			_("Places"),
-			TRUE);
+			_("Places"));
 
 	menuitem->priv->use_image = use_image;
 
@@ -925,8 +937,7 @@ panel_desktop_menu_item_new (gboolean use_image,
 	setup_menuitem (GTK_WIDGET (menuitem),
 			image ? panel_menu_icon_get_size () : GTK_ICON_SIZE_INVALID,
 			image,
-			_("System"),
-			TRUE);
+			_("System"));
 
 	menuitem->priv->use_image = use_image;
 
