@@ -34,6 +34,7 @@
 #include "applet.h"
 #include "nothing.h"
 #include "xstuff.h"
+#include "panel-config-global.h"
 #include "panel-globals.h"
 #include "launcher.h"
 #include "panel-icon-names.h"
@@ -1410,6 +1411,31 @@ panel_util_get_icon_for_uri_method (const char *uri)
 		return NULL;
 }
 
+static const char *
+panel_util_get_icon_for_uri_known_folders (const char *uri)
+{
+	const char *icon;
+	char       *path;
+	int         len;
+
+	icon = NULL;
+	path = gnome_vfs_get_local_path_from_uri (uri);
+
+	len = strlen (path);
+	if (path[len] == '/')
+		path[len] = '\0';
+
+	if (strcmp (path, g_get_home_dir ()) == 0)
+		icon = PANEL_ICON_HOME;
+	else if (strcmp (path,
+			 g_get_user_special_dir (G_USER_DIRECTORY_DESKTOP)) == 0)
+		icon = PANEL_ICON_DESKTOP;
+
+	g_free (path);
+
+	return icon;
+}
+
 /* This is based on nautilus_compute_title_for_uri() and
  * nautilus_file_get_display_name_nocopy() */
 char *
@@ -1508,8 +1534,42 @@ panel_util_get_icon_for_uri (const char *text_uri)
 	if (!g_str_has_prefix (text_uri, "file:"))
 		return NULL;
 
+	icon = panel_util_get_icon_for_uri_known_folders (text_uri);
+	if (icon)
+		return g_strdup (icon);
+
 	return gnome_icon_lookup_sync (gtk_icon_theme_get_default (),
 				       NULL, text_uri, NULL,
 				       GNOME_ICON_LOOKUP_FLAGS_NONE,
 				       GNOME_ICON_LOOKUP_RESULT_FLAGS_NONE);
 }
+
+static gboolean
+panel_util_query_tooltip_cb (GtkWidget  *widget,
+			     gint        x,
+			     gint        y,
+			     gboolean    keyboard_tip,
+			     GtkTooltip *tooltip,
+			     const char *text)
+{
+	if (!panel_global_config_get_tooltips_enabled ())
+		return FALSE;
+
+	gtk_tooltip_set_text (tooltip, text);
+	return TRUE;
+}
+
+void
+panel_util_set_tooltip_text (GtkWidget  *widget,
+			     const char *text)
+{
+	if (string_empty (text)) {
+		g_object_set (widget, "has-tooltip", FALSE, NULL);
+		return;
+	}
+
+	g_object_set (widget, "has-tooltip", TRUE, NULL);
+	g_signal_connect_data (widget, "query-tooltip",
+			       G_CALLBACK (panel_util_query_tooltip_cb),
+			       g_strdup (text), (GClosureNotify) g_free, 0);
+}
